// Copyright 2021, CGVR, Contact: muehlenb@uni-bremen.de

#version 330 core
layout (location = 0) in vec3 aPos; // the position variable has attribute position 0
layout (location = 1) in vec3 aNormal; // the normal variable has attribute position 1
layout (location = 2) in vec2 aTexCoord; // the texcoord variable has attribute position 2
layout (location = 3) in vec3 aTangent;

out vec4 mPos;
out vec3 mNormal;
out vec2 texCoord;
out vec4 mLightPositions[4];
out vec4 mLightFragPosLightSpace[4];
out vec3 mLightDir;

out mat3 TBN;

uniform mat4 projection;
uniform mat4 view;
uniform mat4 model;
uniform mat4 normalMatrix;
uniform float texCoordScale;

uniform mat4 lightSpaceMatrix[4];

void main()
{
	mPos = view * model * vec4(aPos, 1.0);
	
	// Normalize the vector (because of scaling):
	mNormal = normalize(mat3(normalMatrix) * aNormal);
	
	mLightPositions[0] = view * vec4( 2.0, 1.8, 2.0, 1.0);
	mLightPositions[1] = view * vec4( 2.0, 1.8, -2.0, 1.0 );
	mLightPositions[2] = view * vec4( -2.0, 1.8, -2.0, 1.0 );
	mLightPositions[3] = view * vec4( -2.0, 1.8, 2.0, 1.0 );
	
	for(int i=0; i < 4; ++i){
		mLightFragPosLightSpace[i] = lightSpaceMatrix[i] * model * vec4(aPos, 1.0);
	}
	
	mLightDir = normalize(mat3(normalMatrix) * vec3(0, -1, 0));
	
	texCoord = aTexCoord * texCoordScale;
	
    vec3 T = normalize(vec3(view * model * vec4(aTangent, 0.0)));
	vec3 N = normalize(vec3(view * model * vec4(aNormal, 0.0)));
	vec3 B = cross(N, T);
	
	TBN = mat3(T, B, N);
	
    gl_Position = projection * mPos;
}